
static void
add_feed_by_url_async_cb (GObject      *object,
                          GAsyncResult *result,
                          gpointer      user_data)
{
    g_return_if_fail (TRACKER_SPARQL_IS_CONNECTION (object));
    g_return_if_fail (G_IS_ASYNC_RESULT (result));

    TrackerSparqlConnection *connection = TRACKER_SPARQL_CONNECTION (object);
    GError *error = NULL;

    tracker_sparql_connection_update_finish (connection, result, &error);

    if (error != NULL)
    {
        g_warning ("Unable to add feed to Tracker database: %s", error->message);
        g_error_free (error);
        return;
    }
}

static void
add_feed_by_url_async (GpodManager  *self,
                       gchar        *feed_url,
                       GCancellable *cancellable)
{
    g_return_if_fail (GPOD_IS_MANAGER (self));
    g_return_if_fail (feed_url != NULL);
    g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));

    GpodManagerPrivate *priv = gpod_manager_get_instance_private (self);
    GString *query = g_string_new (NULL);

    g_string_printf (query,
                     "INSERT { "
                     "    _:FeedSettings a mfo:FeedSettings ; "
                     "        mfo:updateInterval %i . "
                     "    _:Feed a nie:DataObject, mfo:FeedChannel ; "
                     "        mfo:feedSettings _:FeedSettings ; "
                     "        nie:url <%s> "
                     "}",
                     30, feed_url); // 30 minutes

    tracker_sparql_connection_update_async (priv->tracker,
                                            query->str,
                                            G_PRIORITY_DEFAULT,
                                            cancellable,
                                            add_feed_by_url_async_cb,
                                            NULL);

    g_string_free (query, TRUE);
}

GpodFeedItem *
get_feed_channel_item (GpodManager *self,
                       gchar       *guid)
{
    g_return_val_if_fail (GPOD_IS_MANAGER (self), NULL);
    g_return_val_if_fail (guid != NULL, NULL);

    GpodFeedItem *item = NULL;

    return item;
}

GpodFeedItem *
get_feed_channel_item_by_search (GpodManager *self,
                                 gchar       *search_text)
{
    g_return_val_if_fail (GPOD_IS_MANAGER (self), NULL);
    g_return_val_if_fail (search_text != NULL, NULL);

    GpodFeedItem *item = NULL;

    return item;
}

GList *
get_feed_channel_items (GpodManager *self,
                        gchar       *guid)
{
    g_return_val_if_fail (GPOD_IS_MANAGER (self), NULL);
    g_return_val_if_fail (guid != NULL, NULL);

    GList *items = NULL;

    return items;
}

GList *
get_feed_channel_items_by_url (GpodManager *self,
                               gchar       *feed_url)
{
    g_return_val_if_fail (GPOD_IS_MANAGER (self), NULL);
    g_return_val_if_fail (feed_url != NULL, NULL);

    GList *items = NULL;
    gchar *guid = NULL;

    items = get_feed_channel_items (self, guid);

    g_free (guid);

    return items;
}

GpodFeedChannel *
get_feed_channel (GpodManager *self,
                  gchar       *guid)
{
    g_return_val_if_fail (GPOD_IS_MANAGER (self), NULL);
    g_return_val_if_fail (guid != NULL, NULL);

    GpodFeedChannel *channel = NULL;

    return channel;
}

GpodFeedChannel *
get_feed_channel_by_url (GpodManager *self,
                         gchar       *feed_url)
{
    g_return_val_if_fail (GPOD_IS_MANAGER (self), NULL);
    g_return_val_if_fail (feed_url != NULL, NULL);

    GpodFeedChannel *channel = NULL;
    gchar *guid = NULL;

    channel = get_feed_channel (self, guid);

    g_free (guid);

    return channel;
}




// TODO: Add GError
static void
get_feed (GpodManager  *self,
          gchar        *feed_url,
          GCancellable *cancellable)
{
    g_return_if_fail (GPOD_IS_MANAGER (self));
    g_return_if_fail (feed_url != NULL);
    g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));

    GpodManagerPrivate *priv = gpod_manager_get_instance_private (self);
    GString *query = g_string_new (NULL);
    GError *error = NULL;
    TrackerSparqlCursor *cursor;
    GpodFeedChannel *channel;

    g_string_printf (query,
                     "SELECT "
                     "    ?urn AS urn"
                     "    ?title AS title "
                     "    ?description AS description "
                     "    ?copyright AS copyright "
                     "    ?updated AS updated "
                     "    ?image AS image "
                     "    ?updateInterval AS updateInterval "
                     "WHERE { "
                     "    ?urn a mfo:FeedChannel ; "
                     "        nie:url <%s> ; "
                     "        nie:title ?title ; "
                     "        nie:description ?description ; "
                     "        nie:copyright ?copyright ; "
                     "        nmo:lastMessageDate ?updated ; "
                     "        mfo:image ?image . "
                     "    OPTIONAL { "
                     "        ?urnSettings a mfo:FeedSettings ; "
                     "            mfo:updateInterval ?updateInterval . "
                     "    } "
                     "} "
                     "LIMIT 1", feed_url);

    g_print ("-------------------------------------------------\n");
    g_print ("Feed for: %s\n", feed_url);
    g_print ("-------------------------------------------------\n");
    g_print ("%s\n", query->str);
    g_print ("-------------------------------------------------\n");

    cursor = tracker_sparql_connection_query (priv->tracker, query->str, cancellable, &error);

    g_string_free (query, TRUE);

    if (error != NULL)
    {
        g_warning ("Unable to get feed from Tracker database: %s", error->message);
        g_error_free (error);
        return;
    }

    if (!cursor)
    {
        g_print ("%s\n", "No results were found");
        return;
    }

    g_print ("-------------------------------------------------\n");
    g_print ("Feed Info (%i):\n", tracker_sparql_cursor_get_n_columns (cursor));
    g_print ("-------------------------------------------------\n");

    tracker_sparql_cursor_next (cursor, NULL, NULL);

    channel = gpod_feed_channel_new ();
    channel->guid = tracker_sparql_cursor_get_string (cursor, 0, NULL);
    channel->title = tracker_sparql_cursor_get_string (cursor, 1, NULL);
    channel->description = tracker_sparql_cursor_get_string (cursor, 2, NULL);
    channel->copyright = tracker_sparql_cursor_get_string (cursor, 3, NULL);
    channel->updated = tracker_sparql_cursor_get_string (cursor, 4, NULL);
    channel->image = tracker_sparql_cursor_get_string (cursor, 5, NULL);

    g_print ("Result:\n    %s\n    %s\n    %s\n    %s\n    %s\n    %s\n\n",
             channel->guid,
             channel->title,
             channel->description,
             channel->copyright,
             channel->updated,
             channel->image);

    /*
    g_print ("Result:\n    %s\n    %s\n    %s\n    %s\n    %s\n    %s\n    %s\n\n",
             tracker_sparql_cursor_get_string (cursor, 0, NULL),
             tracker_sparql_cursor_get_string (cursor, 1, NULL),
             tracker_sparql_cursor_get_string (cursor, 2, NULL),
             tracker_sparql_cursor_get_string (cursor, 3, NULL),
             tracker_sparql_cursor_get_string (cursor, 4, NULL),
             tracker_sparql_cursor_get_string (cursor, 5, NULL),
             tracker_sparql_cursor_get_string (cursor, 6, NULL));
    */
    g_print ("-------------------------------------------------\n");

    while (tracker_sparql_cursor_next (cursor, NULL, NULL))
    {
        g_print ("Result:\n    %s\n    %s\n    %s\n    %s\n    %s\n    %s\n    %s\n\n",
                 tracker_sparql_cursor_get_string (cursor, 0, NULL),
                 tracker_sparql_cursor_get_string (cursor, 1, NULL),
                 tracker_sparql_cursor_get_string (cursor, 2, NULL),
                 tracker_sparql_cursor_get_string (cursor, 3, NULL),
                 tracker_sparql_cursor_get_string (cursor, 4, NULL),
                 tracker_sparql_cursor_get_string (cursor, 5, NULL),
                 tracker_sparql_cursor_get_string (cursor, 6, NULL));
        g_print ("-------------------------------------------------\n");
    }

    // TODO: Create GpodFeed struct wrapper type
    // GpodFeed *feed = g_new (GpodFeed, 1);

    g_object_unref (cursor);
}

/*
 *     while (tracker_sparql_cursor_next (cursor, NULL, NULL))
        g_print ("Feed: %s\n", tracker_sparql_cursor_get_string (cursor, 0, NULL));
 */





/*
static void
get_feed_async_cb (GObject      *object,
                   GAsyncResult *result,
                   gpointer      user_data)
{
    g_return_if_fail (TRACKER_SPARQL_IS_CONNECTION (object));
    g_return_if_fail (G_IS_ASYNC_RESULT (result));

    TrackerSparqlConnection *connection = TRACKER_SPARQL_CONNECTION (object);
    GError *error = NULL;

    tracker_sparql_connection_query_finish (connection, result, &error);

    if (error != NULL)
    {
        g_warning ("Unable to get feed from Tracker database: %s", error->message);
        g_error_free (error);
        return;
    }
    else
    {
        tracker_sparql_cursor_next_async (cursor,
                                          query_data->cancellable,
                                          on_cursor_cb,
                                          query_data);
    }
}

static void
get_feed_async (GpodManager  *self,
                gchar        *feed_url,
                GCancellable *cancellable)
{
    g_return_if_fail (GPOD_IS_MANAGER (self));
    g_return_if_fail (feed_url != NULL);
    g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));

    GpodManagerPrivate *priv = gpod_manager_get_instance_private (self);
    GString *query = g_string_new (NULL);

    g_string_printf (query,
                     "SELECT "
                     "nie:title(?feed) "
                     "WHERE { "
                     "    ?feed a mfo:FeedMessage ; "
                     "        nie:url <%s>"
                     "}", feed_url);

    tracker_sparql_connection_query_async (priv->tracker,
                                           query->str,
                                           cancellable,
                                           get_feed_async_cb,
                                           NULL);

    g_string_free (query, TRUE);
}
*/
/*

static void
on_cursor_cb (GObject      *object,
              GAsyncResult *result,
              gpointer      user_data)
{

}

static void
get_feeds_async_cb (GObject      *object,
                    GAsyncResult *result,
                    gpointer      user_data)
{
    TrackerSparqlConnection *connection;
    TrackerSparqlCursor *cursor;
    GError *error = NULL;
    FeedGetQueryData *query_data;

    g_return_if_fail (TRACKER_SPARQL_IS_CONNECTION (object));

    query_data = user_data;

    cursor = tracker_sparql_connection_query_finish (connection,
                                                     result,
                                                     &error);

    if (error != NULL)
    {
        g_warning ("Error on fetching feeds: %s", error->message);

        //gpod_feed_view_add ();
        g_free (query_data);
    }
    else
    {
        tracker_sparql_cursor_next_async (cursor,
                                          query_data->cancellable,
                                          on_cursor_cb,
                                          query_data);
    }
}

static void
get_feeds_async (GpodManager  *self,
                 GList        *list,
                 GCancellable *cancellable)
{
    g_return_if_fail (GPOD_IS_MANAGER (self));

    GpodManagerPrivate *priv = gpod_manager_get_instance_private (self);
    GString *query = NULL;
    FeedGetQueryData *query_data = NULL;

    g_string_printf (query,
                     "SELECT "
                     "nie:url(?feed)"
                     "nie:title(?feed) "
                     "WHERE { "
                     "    ?feed a mfo:FeedChannel "
                     "} "
                     "ORDER BY nie:title(?feed)");

    query_data = g_new (FeedGetQueryData, 1);
    query_data->feeds = list;
    query_data->cancellable = cancellable;

    tracker_sparql_connection_query_async (priv->tracker,
                                           query->str,
                                           query_data->cancellable,
                                           get_feeds_async_cb,
                                           query_data);

    g_string_free (query, TRUE);
}
*/

