#define G_LOG_DOMAIN "gpod-feed-item"

#include "gpod-feed-item.h"

#include <glib.h>

/**
 * SECTION:gpod-feed-item
 * @short_description: a container to hold a RSS item in a #GpodChannel
 *
 * #GpodFeedItem is the representation of a RSS item in a valid RSS feed. It
 * contains information about the item like title, link, publish date,
 * enclosure, etc..
 *
 * Since a #GpodChannel can contain many #GpodFeedItems it is important to manage
 * the creation (ref) and deletion (unref) of #GpodFeedItem. You can control the
 * lifetime of a #GpodFeedItem using gpod_feed_item_ref() and gpod_feed_item_unref().
 */

G_DEFINE_BOXED_TYPE (GpodFeedItem, gpod_feed_item, gpod_feed_item_ref, gpod_feed_item_unref)

/**
 * gpod_feed_item_new: (constructor)
 *
 * Creates a new #GpodFeedItem, a container to hold a RSS item in a #GpodChannel.
 * #GpodFeed.
 *
 * Returns: (transfer full): the newly created #GpodFeedItem
 */
GpodFeedItem *
gpod_feed_item_new (void)
{
    GpodFeedItem *object;

    object = g_slice_new0 (GpodFeedItem);
    object->ref_count = 1;

    object->title = NULL;
    object->description = NULL;
    object->link = NULL;
    object->publish_date = NULL;
    object->enclosure = NULL;
    object->guid = NULL;

    return object;
}

/**
 * gpod_feed_item_ref:
 * @object: a #GpodFeedItem
 *
 * Increase by one the reference count of a #GpodFeedItem.
 *
 * Returns: (transfer none): the passed #GpodFeedItem, with the reference count
 *                           increased by one.
 */
GpodFeedItem *
gpod_feed_item_ref (GpodFeedItem *object)
{
    g_return_val_if_fail (object != NULL, NULL);
    g_return_val_if_fail (object->ref_count > 0, NULL);

    object->ref_count++;

    return object;
}

/**
 * gpod_feed_item_unref:
 * @object: a #GpodFeedItem
 *
 * Decreases by one the reference count of a #GpodFeedItem. If the
 * reference count reaches zero, the object is destroyed and all
 * its allocated resources are freed.
 */
void
gpod_feed_item_unref (GpodFeedItem *object)
{
    g_return_if_fail (object != NULL);
    g_return_if_fail (object->ref_count > 0);

    if (--object->ref_count == 0)
    {
/*
        if (object->title != NULL)
            g_free (object->title);

        if (object->description != NULL)
            g_free (object->description);

        if (object->link != NULL)
            g_free (object->link);

        if (object->publish_date != NULL)
            g_free (object->publish_date);

        if (object->enclosure != NULL)
            g_free (object->enclosure);

        if (object->guid != NULL)
            g_free (object->guid);

        object->title = NULL;
        object->description = NULL;
        object->link = NULL;
        object->publish_date = NULL;
        object->enclosure = NULL;
        object->guid = NULL;
*/
        g_slice_free (GpodFeedItem, object);
    }
}
