#define G_LOG_DOMAIN "gpod-tracker"

#include "config.h"

#include "gpod-tracker.h"

#include <tracker-sparql.h>

#include <glib.h>
#include <glib-object.h>

/**
 * SECTION:gpod-tracker
 * @title: GpodTracker
 * @short_description: #GpodTracker helps to create connections to Tracker and
 *                     get/set data in tracker asynchronously.
 * @include: gpod-tracker.h
 *
 * #GpodTracker helps to create connections to Tracker and get/set data in
 * tracker asynchronously.
 *
 * ## Usage
 * |[
 *
 * ]|
 **/

typedef struct _GpodTrackerPrivate
{
    TrackerSparqlConnection *connection;
} GpodTrackerPrivate;

struct _GpodTracker
{
    GObject parent;
};

G_DEFINE_TYPE_WITH_PRIVATE (GpodTracker, gpod_tracker, G_TYPE_OBJECT)

static void
connection_async_cb (GObject      *object,
                     GAsyncResult *result,
                     gpointer      user_data)
{
    g_return_if_fail (GPOD_IS_TRACKER (user_data));

    GpodTracker *self = GPOD_TRACKER (user_data);
    GpodTrackerPrivate *priv = gpod_tracker_get_instance_private (self);
    GError *error = NULL;

    priv->connection = tracker_sparql_connection_get_finish (result, &error);

    if (error)
    {
        g_warning ("Error connecting to tracker: %s", error->message);
        g_error_free (error);
    }
}

static void
connection_async (GpodTracker         *self,
                  GCancellable        *cancellable,
                  GAsyncReadyCallback  callback)
{
    g_return_if_fail (GPOD_IS_TRACKER (self));
    g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));

    tracker_sparql_connection_get_async (cancellable, callback, self);
}

static void
connection_init (GpodTracker  *self,
                 GCancellable *cancellable)
{
    connection_async (self, cancellable, connection_async_cb);
}

static gboolean
check_database (GpodTracker  *self)
{
    GpodTrackerPrivate *priv = gpod_tracker_get_instance_private (self);

    if (priv->connection)
        return TRUE;
    else
        return FALSE;
}

static void
query_async_update_finish (GObject       *object,
                           GAsyncResult  *result,
                           gpointer       user_data,
                           GError       **error)
{
    g_return_if_fail (TRACKER_SPARQL_IS_CONNECTION (object));
    g_return_if_fail (G_IS_ASYNC_RESULT (result));

    TrackerSparqlConnection *connection = TRACKER_SPARQL_CONNECTION (object);
    GError *internal_error = NULL;

    tracker_sparql_connection_update_finish (connection, result, &internal_error);

    if (error != NULL)
    {
        g_propagate_error (error, internal_error);
        return;
    }
}

static void
query_async_update_cb (GObject      *object,
                       GAsyncResult *result,
                       gpointer      user_data)
{
    g_return_if_fail (TRACKER_SPARQL_IS_CONNECTION (object));
    g_return_if_fail (G_IS_ASYNC_RESULT (result));

    GError *error = NULL;

    query_async_update_finish (object, result, user_data, &error);

    if (error != NULL)
    {
        g_warning ("Unable to add data to Tracker database: %s", error->message);
        g_error_free (error);
        return;
    }
}

static void
query_async_update_full (GpodTracker         *self,
                         gchar               *query,
                         GCancellable        *cancellable,
                         GAsyncReadyCallback  callback)
{
    g_return_if_fail (GPOD_IS_TRACKER (self));
    g_return_if_fail (query != NULL);
    g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));

    GpodTrackerPrivate *priv = gpod_tracker_get_instance_private (self);

    if (!check_database (self))
    {
        g_print ("No database connection.");
        return;
    }

    tracker_sparql_connection_update_async (priv->connection,
                                            query,
                                            G_PRIORITY_DEFAULT,
                                            cancellable,
                                            callback,
                                            NULL);
}

static void
query_async_update (GpodTracker  *self,
                    gchar        *query,
                    GCancellable *cancellable)
{
    g_return_if_fail (GPOD_IS_TRACKER (self));
    g_return_if_fail (query != NULL);
    g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));

    query_async_update_full (self, query, cancellable, query_async_update_cb);
}

static gboolean
query_async_cursor_finish (TrackerSparqlCursor  *cursor,
                           GAsyncResult         *result,
                           GError              **error)
{
    g_return_val_if_fail (TRACKER_SPARQL_IS_CURSOR (cursor), FALSE);
    g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);

    gboolean results;
    GError *internal_error = NULL;

    results = tracker_sparql_cursor_next_finish (cursor, result, &internal_error);

    if (internal_error != NULL)
    {
        g_propagate_error (error, internal_error);
        return FALSE;
    }

    return results;
}

static void
query_async_cursor_next (TrackerSparqlCursor *cursor,
                         GCancellable        *cancellable,
                         GAsyncReadyCallback  callback,
                         gpointer             user_data)
{
    g_return_if_fail (TRACKER_SPARQL_IS_CURSOR (cursor));
    g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));

    tracker_sparql_cursor_next_async (cursor, cancellable, callback, user_data);
}

static TrackerSparqlCursor *
query_async_finish (TrackerSparqlConnection  *connection,
                    GAsyncResult             *result,
                    gpointer                  user_data,
                    GError                  **error)
{
    g_return_val_if_fail (TRACKER_SPARQL_IS_CONNECTION (connection), NULL);
    g_return_val_if_fail (G_IS_ASYNC_RESULT (result), NULL);

    TrackerSparqlCursor *cursor;
    GError *internal_error = NULL;

    cursor = tracker_sparql_connection_query_finish (connection, result, &internal_error);

    if (internal_error != NULL)
    {
        g_propagate_error (error, internal_error);
        return NULL;
    }

    return cursor;
}

static void
query_async (GpodTracker         *self,
             gchar               *query,
             GCancellable        *cancellable,
             GAsyncReadyCallback  callback,
             gpointer             user_data)
{
    g_return_if_fail (GPOD_IS_TRACKER (self));
    g_return_if_fail (query != NULL);
    g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));

    GpodTrackerPrivate *priv = gpod_tracker_get_instance_private (self);
    GpodTrackerQuery *query_data = g_slice_new0 (GpodTrackerQuery);

    if (!check_database (self))
    {
        g_print ("No database connection.");
        return;
    }

    query_data->cancellable = cancellable;
    query_data->user_data = user_data;

    // TODO: tracker_sparql_escape_string (query)
    tracker_sparql_connection_query_async (priv->connection,
                                           query,
                                           cancellable,
                                           callback,
                                           query_data);
}

/*
 * gpod_tracker_finalize:
 * @object: An instance of #GpodTracker
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
gpod_tracker_finalize (GObject *object)
{
    GpodTracker *self = GPOD_TRACKER (object);
    GpodTrackerPrivate *priv = gpod_tracker_get_instance_private (self);

    g_clear_object (&priv->connection);

    G_OBJECT_CLASS (gpod_tracker_parent_class)->finalize(object);
}

/*
 * gpod_tracker_init:
 * @self: An instance of #GpodTracker
 *
 * Initializes various aspects of the #GpodTracker. Anything the #GpodTracker
 * would need before becoming presented should be done here.
 */
static void
gpod_tracker_init (GpodTracker *self)
{
#ifdef TRACKER_IS_VERSION_2
    //TODO: Set a dconf flag
    // tracker_sparql_connection_set_domain (PACKAGE_ID);
#endif

    // TODO: Switch to async connect
    //connection_init (self, NULL);

    //return;

    GpodTrackerPrivate *priv = gpod_tracker_get_instance_private (self);
    GError *error = NULL;

    priv->connection = tracker_sparql_connection_get (NULL, &error);

    if (error)
    {
        g_warning ("Error connecting to tracker: %s", error->message);
        g_error_free (error);
        return;
    }
}

/*
 * gpod_tracker_class_init:
 * @klass: An instance of #GpodTrackerClass
 *
 * Initializes the various virtural functions in #GpodTrackerClass. In
 * addtition it connects the various #GtkWidgets from the gresource file using
 * gtk_widget_class_set_template_from_resource().
 */
static void
gpod_tracker_class_init (GpodTrackerClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->finalize = gpod_tracker_finalize;
}

/**
 * gpod_tracker_new:
 *
 * Creates a new #GpodTracker, starts a Tracker session and allows queries to
 * Tracker.
 *
 * ## Usage
 * |[
 * GpodTracker *tracker;
 *
 * tracker = gpod_tracker_new ();
 * ]|
 *
 * Returns: #GpodTracker
 */
GpodTracker *
gpod_tracker_new ()
{
    return g_object_new (GPOD_TYPE_TRACKER,
                         NULL);
}

/**
 * gpod_tracker_check_database:
 * @self: a #GpodTracker
 *
 * Returns: if the database connection is made
 */
gboolean
gpod_tracker_check_database (GpodTracker *self)
{
    return check_database (self);
}

/**
 * gpod_tracker_connection_init:
 * @self: a #GpodTracker
 * @cancellable: a #GCancellable
 *
 * Starts a connection the Tracker database
 */
void
gpod_tracker_connection_init (GpodTracker  *self,
                              GCancellable *cancellable)
{
    connection_init (self, cancellable);
}

/**
 * gpod_tracker_query_update_finish:
 * @object: a #TrackerSparqlConnection
 * @result: the results from the Tracker database
 * @user_data: user data passed when the query was sent, will be a
 *             #GpodTrackerQuery struct
 * @error: a #GError
 *
 * Finishes the inital query from the @object, returning @results. This function
 * should be called inside of function passed as the #GAsyncReadyCallback of
 * gpod_tracker_query_update_full() or gpod_tracker_query_update().
 *
 * @error is optional, but recommended.
 */
void
gpod_tracker_query_update_finish (GObject       *object,
                                  GAsyncResult  *result,
                                  gpointer       user_data,
                                  GError       **error)
{
    query_async_update_finish (object, result, user_data, error);
}

/**
 * gpod_tracker_query_update_full:
 * @self: a #GpodTracker
 * @query: the string to query
 * @cancellable: a #GCancellable
 * @callback: the callback function which calls
 *            gpod_tracker_query_update_finish()
 *
 * Executes a @query update on the tracker database. A @callback must be given
 * which calls gpod_tracker_query_update_finish(), allowing the query to be
 * asynchronous.
 *
 * @cancellable is optional.
 */
void
gpod_tracker_query_update_full (GpodTracker         *self,
                                gchar               *query,
                                GCancellable        *cancellable,
                                GAsyncReadyCallback  callback)
{
    query_async_update_full (self, query, cancellable, callback);
}

/**
 * gpod_tracker_query_update:
 * @self: a #GpodTracker
 * @query: the string to query
 * @cancellable: a #GCancellable
 *
 * Same as gpod_tracker_query_update_full() but uses an internal
 * #GAsyncReadyCallback. Allowing the @query to update the Tracker database with
 * out having to write more asynchronous code.
 *
 * @cancellable is optional.
 */
void
gpod_tracker_query_update (GpodTracker  *self,
                           gchar        *query,
                           GCancellable *cancellable)
{
    query_async_update (self, query, cancellable);
}

/**
 * gpod_tracker_query_finish:
 * @cursor: a #TrackerSparqlCursor
 * @result: the results from the Tracker database
 * @error: a #GError
 *
 * Finishes the async operation from the Tracker @cursor, started in
 * gpod_tracker_query_cursor_next(). This function should be called inside of
 * the function passed as the #GAsyncReadyCallback of
 * gpod_tracker_query_cursor_next().
 *
 * @error is optional, but recommended.
 */
gboolean
gpod_tracker_query_cursor_finish (TrackerSparqlCursor  *cursor,
                                  GAsyncResult         *result,
                                  GError              **error)
{
    return query_async_cursor_finish (cursor, result, error);
}

/**
 * gpod_tracker_query_cursor_next:
 * @cursor: a #TrackerSparqlCursor
 * @cancellable: a #GCancellable
 * @callback: the callback function which calls
 *            gpod_tracker_query_finish()
 * @user_data: (gpointer): data the user wishes to pass into the callback
 *
 * Returns the next entry (@cursor) from the Tracker database. A @callback must
 * be given which calls gpod_tracker_query_cursor_finish(), allowing the cursor
 * request to be asynchronous.
 *
 * @cancellable is optional.
 */
void
gpod_tracker_query_cursor_next (TrackerSparqlCursor *cursor,
                                GCancellable        *cancellable,
                                GAsyncReadyCallback  callback,
                                gpointer             user_data)
{
    query_async_cursor_next (cursor, cancellable, callback, user_data);
}

/**
 * gpod_tracker_query_finish:
 * @connection: a #TrackerSparqlConnection
 * @result: the results from the Tracker database
 * @user_data: user data passed when the query was sent, will be a
 *             #GpodTrackerQuery struct
 * @error: a #GError
 *
 * Finishes the inital query from the @object, returning @results. This function
 * should be called inside of function passed as the #GAsyncReadyCallback of
 * gpod_tracker_query().
 *
 * @error is optional, but recommended.
 */
TrackerSparqlCursor *
gpod_tracker_query_finish (TrackerSparqlConnection  *connection,
                           GAsyncResult             *result,
                           gpointer                  user_data,
                           GError                  **error)
{
    return query_async_finish (connection, result, user_data, error);
}

/**
 * gpod_tracker_query:
 * @self: a #GpodTracker
 * @query: the string to query
 * @cancellable: a #GCancellable
 * @callback: the callback function which calls
 *            gpod_tracker_query_finish()
 * @user_data: (gpointer): data the user wishes to pass into the callback
 *
 * Executes the @query statement on the tracker database. A @callback must be
 * given which calls gpod_tracker_query_finish(), allowing the query to be
 * asynchronous.
 *
 * @cancellable is optional.
 */
void
gpod_tracker_query (GpodTracker         *self,
                    gchar               *query,
                    GCancellable        *cancellable,
                    GAsyncReadyCallback  callback,
                    gpointer             user_data)
{
    query_async (self, query, cancellable, callback, user_data);
}
